"use strict";

Object.defineProperty(exports, "__esModule", { value: true });
function scrollTo(element, target, duration) {
    target = Math.round(target);
    duration = Math.round(duration);
    if (duration < 0) {
        return Promise.reject("bad duration");
    }
    if (duration === 0) {
        element.scrollTop = target;
        return Promise.resolve();
    }
    var startTime = Date.now();
    var endTime = startTime + duration;
    var startTop = element.scrollTop;
    var distance = target - startTop;
    var smoothStep = function smoothStep(start, end, point) {
        if (point <= start) {
            return 0;
        }
        if (point >= end) {
            return 1;
        }
        var x = (point - start) / (end - start);
        return x * x * (3 - 2 * x);
    };
    return new Promise(function (resolve, reject) {
        var previousTop = element.scrollTop;
        var scrollFrame = function scrollFrame() {
            if (element.scrollTop !== previousTop) {
                return;
            }
            var now = Date.now();
            var point = smoothStep(startTime, endTime, now);
            var frameTop = Math.round(startTop + distance * point);
            element.scrollTop = frameTop;
            if (now >= endTime) {
                resolve();
                return;
            }
            if (element.scrollTop === previousTop && element.scrollTop !== frameTop) {
                resolve();
                return;
            }
            previousTop = element.scrollTop;
            setTimeout(scrollFrame, 0);
        };
        setTimeout(scrollFrame, 0);
    });
}
exports.scrollTo = scrollTo;
//# sourceMappingURL=scroll-to.js.map